import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {Constants} from "src/utils/constants";
import {CreateBlogPostDto} from "./dto/create-blog-post.dto";
import {UpdateBlogPostDto} from "./dto/update-blog-post.dto";
import {BlogPost} from "./entities/blog-post.entity";
import {BlogPostRepository} from "./blog-post.repository";
import {ResponseUtils} from "src/utils/response.utils";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";
import {DoSpacesService} from "src/modules/SpacesModule/SpacesService/doSpacesService";
import {urlSlug} from "src/helper/common-helper";

@Injectable()
export class BlogPostService {
    constructor(
        private readonly repository: BlogPostRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateBlogPostDto,
        files: {
            photo?: UploadedMulterFileI,
            authorImage?: UploadedMulterFileI,
        }
    ): Promise<any | Error> {
        if (files && files.photo) {
            const photo: any = await this.doSpaceService.uploadFile(files.photo[0], "Blog/");
            dto.photo = photo;
        } else dto.photo = null;

        if (files && files.authorImage) {
            const authorImage: any = await this.doSpaceService.uploadFile(files.authorImage[0], "BlogAuthor/");
            dto.authorImage = authorImage;
        } else dto.authorImage = null;

        let slug = urlSlug(dto.title);
        dto.urlSlug = slug;

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Blog post created successfully", "data", data);
    }

    async findAll(): Promise<BlogPost[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Blog post list fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<any | Error> {
        let data: any = await this.repository.findOneEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (data) {
            let viewCount = 0;
            let totalViews = data[0].totalViews;
            let postId = data[0].id;
            viewCount = ((totalViews ? totalViews : 0) - 0) + (1);
            let blogPostDto = {
                "totalViews": viewCount
            };
            await this.repository.updateBlogViewCount(postId, blogPostDto as UpdateBlogPostDto);
        }
        return ResponseUtils.successResponseHandler(200, "Blog post fetched successfully", "data", data);
    }

    async update(
        id: string,
        dto: UpdateBlogPostDto,
        files: {
            photo?: UploadedMulterFileI,
            authorImage?: UploadedMulterFileI,
        }
    ): Promise<BlogPost | Error> {
        if (files && files.photo) {
            const photo: any = await this.doSpaceService.uploadFile(files.photo[0], "Blog/");
            dto.photo = photo;
        }
        if (files && files.authorImage) {
            const authorImage: any = await this.doSpaceService.uploadFile(files.authorImage[0], "BlogAuthor/");
            dto.authorImage = authorImage;
        }
        let slug = urlSlug(dto.title);
        dto.urlSlug = slug;
        const data = await this.repository.updateBlogPost(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Blog post updated successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Blog post deleted successfully");
    }
}
